#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Program = require("commander");
var git_1 = require("../git");
var local_storage_1 = require("../local-storage");
var release_1 = require("../release");
/**
 Release CLI.
 */
Program
    .command('bump <type>')
    .description('Bumps the current release of the tutorial')
    .option('-m, --message [message]', 'A message describing the newly created release')
    .action(function (type, options) {
    local_storage_1.localStorage.assertTortilla(true);
    release_1.Release.bump(type, options);
});
Program
    .command('revert')
    .description('Reverts the most recent commit')
    .action(function () {
    local_storage_1.localStorage.assertTortilla(true);
    release_1.Release.revert();
});
Program
    .command('list [branch]')
    .description('Lists all releases for given branch')
    .action(function (branch) {
    if (branch === void 0) { branch = git_1.Git.activeBranchName(); }
    local_storage_1.localStorage.assertTortilla(true);
    var releases = release_1.Release.all(null, branch)
        .map(function (release) { return branch + "@" + release_1.Release.format(release); });
    releases[0] += ' -> current release';
    console.log(releases.join('\n'));
});
Program
    .command('current')
    .description('Prints the current release')
    .action(function () {
    local_storage_1.localStorage.assertTortilla(true);
    release_1.Release.printCurrent();
});
Program
    .command('diff <sourceRelease> <destinationRelease>')
    .description('Runs `git diff` between 2 specified releases')
    .allowUnknownOption(true)
    .action(function (sourceRelease, destinationRelease) {
    local_storage_1.localStorage.assertTortilla(true);
    release_1.Release.diff(sourceRelease, destinationRelease, process.argv.slice(5));
});
Program.parse(process.argv);
//# sourceMappingURL=tortilla-release.js.map