#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Program = require("commander");
var open = require("open");
var semver = require("semver");
var Pack = require("../../package.json");
var essentials_1 = require("../essentials");
var git_1 = require("../git");
var local_storage_1 = require("../local-storage");
if (!semver.gt(process.version, '6.0.0')) {
    /* tslint:disable-next-line */
    require('babel-register');
}
/**
  CLI entry point.
 */
Program
    .version(Pack.version)
    .description(Pack.description);
Program
    .command('create [name]')
    .description('Creates a new Tortilla project with the provided name')
    .option('-o, --output [path]', 'The output path of the newly created project')
    .option('-m, --message [message]', "The created project's initial commit's message")
    .option('--override', 'Override project directory if already exists')
    .action(function (name, options) {
    essentials_1.Essentials.create(name, options);
});
Program
    .command('init [name]')
    .description('Initializes Tortilla essentials in the provided project')
    .action(function (dir) {
    var localStorage = dir ? local_storage_1.localStorage.create(dir) : local_storage_1.localStorage;
    localStorage.assertTortilla();
    essentials_1.Essentials.ensure(dir);
});
Program
    .command('dispose [cwd]')
    .description('Remove tortilla from project')
    .action(function (cwd) {
    local_storage_1.localStorage.assertTortilla(true);
    essentials_1.Essentials.dispose(cwd);
    console.log('Tortilla successfully disposed');
});
Program
    .command('push <remote> <branch>')
    .description('Push tutorial <branch> and related git-assets to <remote>')
    .action(function (remote, branch) {
    git_1.Git.pushTutorial(remote, branch);
});
Program
    .command('pull <remote> <branch>')
    .description('Pull tutorial <branch> and related git-assets from <remote>')
    .action(function (remote, branch) {
    git_1.Git.pullTutorial(remote, branch);
});
Program
    .command('review <remote> <branch>')
    .option('-p, --print [print]', 'Print only, do not open in browser')
    .description('Print a comparison URL between original branch currently changed branch')
    .action(function (remote, branch, options) {
    var url = git_1.Git.reviewTutorial(remote, branch);
    if (options.print) {
        console.log(url);
    }
    else {
        console.log('Opening...');
        open(git_1.Git.reviewTutorial(remote, branch));
    }
});
Program
    .command('status')
    .description('Print edit status followed by git-status')
    .option('-i, --instruct [instruct]', 'Print additional instructions')
    .action(function (options) {
    git_1.Git.tutorialStatus(options);
});
Program
    .command('dump <command...>', 'Manage dump file')
    .command('manual <command...>', 'Manage manual files')
    .command('package <command...>', 'Manage package.json')
    .command('release <command...>', 'Manage tutorial releases')
    .command('step <command...>', 'Manage step commits history')
    .command('strict <command...>', 'Manage strict mode')
    .command('submodule <command...>', 'Manage submodules');
Program.parse(process.argv);
//# sourceMappingURL=tortilla.js.map