"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Fs = require("fs-extra");
var git_1 = require("../git");
var local_storage_1 = require("../local-storage");
var paths_1 = require("../paths");
var step_1 = require("../step");
/**
 Commit-message git hook launches right after we wrote our commit message.
 If an error was thrown the commit process will be aborted with the provided error
 message.
 */
function run() {
    // Should abort hook once steps limit reached
    if (git_1.Git.rebasing() && local_storage_1.localStorage.getItem('REBASE_HOOKS_DISABLED')) {
        return;
    }
    // Amend is the only thing allowed by tortilla, the rest is irrelevant
    if (!process.env.TORTILLA_CHILD_PROCESS && !git_1.Git.gonnaAmend()) {
        return;
    }
    // If we're amending to the root commit then a step prefix is not needed
    if (git_1.Git.gonnaAmend() && !local_storage_1.localStorage.getItem('HOOK_STEP')) {
        return;
    }
    var commitMessage = Fs.readFileSync(paths_1.Paths.git.messages.commit, 'utf8');
    // Prepend a step prefix to the commit message
    var step = local_storage_1.localStorage.getItem('HOOK_STEP') || step_1.Step.next(1);
    var fixedcommitMessage = "Step " + step + ": " + commitMessage;
    // Clearing storage to prevent conflicts with upcoming commits
    local_storage_1.localStorage.removeItem('HOOK_STEP');
    // Rewrite the commit with a step prefix
    Fs.writeFileSync(paths_1.Paths.git.messages.commit, fixedcommitMessage);
}
run();
//# sourceMappingURL=commit-msg.js.map