"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Fs = require("fs-extra");
var git_1 = require("../git");
var local_storage_1 = require("../local-storage");
var paths_1 = require("../paths");
var step_1 = require("../step");
/**
 Prepare-commit-message git hook launches right before we write our commit message.
 If an error was thrown the commit process will be aborted with the provided error
 message.
 */
function run() {
    // Should abort hook once steps limit reached
    if (git_1.Git.rebasing() && local_storage_1.localStorage.getItem('REBASE_HOOKS_DISABLED')) {
        return;
    }
    // Amend is the only thing allowed by tortilla, the rest is irrelevant
    if (!process.env.TORTILLA_CHILD_PROCESS && !git_1.Git.gonnaAmend()) {
        return;
    }
    // We don't wanna affect cherry-picks done by step editing
    if (git_1.Git.cherryPicking()) {
        return;
    }
    var commitMessage = Fs.readFileSync(paths_1.Paths.git.messages.commit, 'utf8');
    // The descriptor will contain the raw message with no step prefix
    var stepDescriptor = step_1.Step.descriptor(commitMessage);
    // If step not detected we're probably amending to the root commit
    if (!stepDescriptor) {
        return;
    }
    // If gonna amend then the step is already determined
    if (git_1.Git.gonnaAmend() && !local_storage_1.localStorage.getItem('HOOK_STEP')) {
        local_storage_1.localStorage.setItem('HOOK_STEP', stepDescriptor.number);
    }
    // Rewrite the commit message with no step prefix
    Fs.writeFileSync(paths_1.Paths.git.messages.commit, stepDescriptor.message);
}
run();
//# sourceMappingURL=prepare-commit-msg.js.map