"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var child_process = require("child_process");
var dump_1 = require("./dump");
var git_1 = require("./git");
/*
 lib.ts module provides a runtime access to Tortilla's API through a cluster
 to distribute the work to another core and not to block the main thread
**/
// Will call the function through the worker
var callTortillaFn = function (fnName) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    return new Promise(function (resolve, reject) {
        // Note that we use a child_process here and not a cluster since we don't wanna
        // fork the main module
        var worker = child_process.fork(__filename, [], {
            env: __assign({}, process.env, { TORTILLA_WORKER: '1' })
        });
        worker.send({
            op: 'exec',
            fn: fnName,
            args: args,
        });
        // On result
        worker.on('message', function (msg) {
            if (typeof msg !== 'object') {
                return;
            }
            if (msg.op !== 'result') {
                return;
            }
            if (msg.fn !== fnName) {
                return;
            }
            resolve(msg.payload);
        });
        // On error
        worker.on('message', function (msg) {
            if (typeof msg !== 'object') {
                return;
            }
            if (msg.op !== 'error') {
                return;
            }
            if (msg.fn !== fnName) {
                return;
            }
            reject(msg.payload);
        });
    });
};
// Helper function that will simulate a registered user in the git.config
var withUser = function (fn) { return function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var result = '';
    var nameExists = false;
    var emailExists = false;
    // Upcoming method requires user.name and user.email to be set
    try {
        // If invocation was successful it means name exists
        try {
            git_1.Git(['config', '--global', 'user.name']);
            nameExists = true;
        }
        catch (e) {
            git_1.Git(['config', '--global', 'user.name', 'tortilla']);
        }
        // If invocation was successful it means email exists
        try {
            git_1.Git(['config', '--global', 'user.email']);
            emailExists = true;
        }
        catch (e) {
            git_1.Git(['config', '--global', 'user.email', 'tortilla@tortilla.academy']);
        }
        result = fn.apply(void 0, args);
    }
    finally {
        if (!nameExists) {
            git_1.Git(['config', '--global', '--unset', 'user.name']);
        }
        if (!emailExists) {
            git_1.Git(['config', '--global', '--unset', 'user.email']);
        }
    }
    return result;
}; };
exports.diffReleases = withUser(function (dump, srcRelease, destRelease) {
    return dump_1.Dump.diffReleases(dump, srcRelease, destRelease);
});
// In worker
if (!process.env.TORTILLA_WORKER) {
    // Listen for signals
    process.on('message', function (msg) {
        if (msg.op !== 'exec') {
            return;
        }
        var handler;
        switch (msg.fn) {
            case 'diffReleases': {
                handler = exports.diffReleases;
                break;
            }
            // Escape if function name is not supported
            default: return;
        }
        try {
            var result = handler.apply(void 0, msg.args);
            process.send({
                op: 'result',
                fn: msg.fn,
                id: msg.id,
                payload: result,
            });
        }
        catch (e) {
            process.send({
                op: 'error',
                fn: msg.fn,
                id: msg.id,
                payload: e,
            });
        }
    });
}
//# sourceMappingURL=lib.js.map