"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var Path = require("path");
var git_1 = require("../../git");
var paths_1 = require("../../paths");
var step_1 = require("../../step");
var translator_1 = require("../../translator");
var utils_1 = require("../../utils");
var index_1 = require("../index");
/**
  Provides a navigation bar between steps. The navigation bar should be rendered
  dynamically based on the current step we're currently in.
 */
var t = translator_1.Translator.translate.bind(translator_1.Translator);
index_1.Renderer.registerHelper('navStep', function (options) {
    var step = options.hash.step || _this.step || step_1.Step.currentSuper();
    // If there is no belonging step, don't render anything
    if (!step) {
        return '';
    }
    // Editing root
    if (step === 'root') {
        var anySuperStep = !!git_1.Git(['log', 'ORIG_HEAD', '-1', '--grep', '^Step [0-9]\\+:']);
        // If there are no any steps yet, don't show nav bar
        if (!anySuperStep) {
            return '';
        }
        var ref = options.hash.ref || Path.relative(utils_1.Utils.cwd(), Path.resolve(paths_1.Paths.manuals.views, 'step1.md'));
        return index_1.Renderer.renderTemplateFile('nav-step', {
            nextOnly: true,
            text: t('nav.begin'),
            ref: ref,
        });
    }
    // Convert to number just in case, so we can run arbitrary operations
    step = Number(step);
    // Get an array of all super steps in the current tutorial
    var superSteps = git_1.Git([
        'log', git_1.Git.activeBranchName(),
        '--grep', '^Step [0-9]\\+:',
        '--format=%s',
    ]).split('\n')
        .filter(Boolean)
        .map(function (commitMessage) { return commitMessage.match(/^Step (\d+)/)[1]; })
        .map(Number);
    // If there are no super steps at all
    if (superSteps.length === 0) {
        return '';
    }
    // The order is the other way around, this way we save ourselves the sorting
    var recentSuperStep = superSteps[0];
    // If this is the last step
    if (step === recentSuperStep) {
        // If we only have a single step
        if (step === 1) {
            return index_1.Renderer.renderTemplateFile('nav-step', {
                prevOnly: true,
                text: t('nav.intro'),
                ref: options.hash.ref || index_1.Renderer.resolve('../../../README.md'),
            });
        }
        else {
            return index_1.Renderer.renderTemplateFile('nav-step', {
                prevOnly: true,
                text: t('nav.prev'),
                ref: options.hash.ref || index_1.Renderer.resolve("step" + (step - 1) + ".md"),
            });
        }
    }
    // If this is the first super step
    if (step === 1) {
        return index_1.Renderer.renderTemplateFile('nav-step', {
            bidirectional: true,
            nextText: t('nav.next'),
            nextRef: options.hash.nextRef || index_1.Renderer.resolve('step2.md'),
            prevText: t('nav.intro'),
            prevRef: options.hash.prevRef || index_1.Renderer.resolve('../../../README.md'),
        });
    }
    // Any other case
    return index_1.Renderer.renderTemplateFile('nav-step', {
        bidirectional: true,
        nextText: t('nav.next'),
        nextRef: options.hash.nextRef || index_1.Renderer.resolve("step" + (step + 1) + ".md"),
        prevText: t('nav.prev'),
        prevRef: options.hash.prevRef || index_1.Renderer.resolve("step" + (step - 1) + ".md"),
    });
}, {
    mdWrap: true,
});
index_1.Renderer.registerTransformation('medium', 'navStep', function (view) {
    var isPrev = !!view.match('\\|:-');
    var isNext = !!view.match('-:\\|');
    // ⟸ PREVIOUS STEP ║ NEXT STEP ⟹
    if (isPrev && isNext) {
        var prevMatches = view.match(/\[< ([^\]]+)\]\(([^\)]+)\)/);
        var prevText = prevMatches[1];
        var prevRef = prevMatches[2];
        var nextMatches = view.match(/\[([^\]]+) >\]\(([^\)]+)\)/);
        var nextText = nextMatches[1];
        var nextRef = nextMatches[2];
        var prevButton = "\u27F8 <a href=\"" + prevRef + "\">" + prevText.toUpperCase() + "</a>";
        var nextButton = "<a href=\"" + nextRef + "\">" + nextText.toUpperCase() + "</a> \u27F9";
        return prevButton + " <b>\u2551</b> " + nextButton;
    }
    // ⟸ PREVIOUS STEP ║
    if (isPrev) {
        var matches = view.match(/\[< ([^\]]+)\]\(([^\)]+)\)/);
        var text = matches[1];
        var ref = matches[2];
        return "\u27F8 <a href=\"" + ref + "\">" + text.toUpperCase() + "</a> <b>\u2551</b>";
    }
    // ║ NEXT STEP ⟹
    if (isNext) {
        var matches = view.match(/\[([^\]]+) >\]\(([^\)]+)\)/);
        var text = matches[1];
        var ref = matches[2];
        return "<b>\u2551</b> <a href=\"" + ref + "\">" + text.toUpperCase() + "</a> \u27F9";
    }
    return view;
});
//# sourceMappingURL=nav-step.js.map