"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var i18next_1 = require("i18next");
var Path = require("path");
var paths_1 = require("../paths");
var utils_1 = require("../utils");
var translation_1 = require("./translation");
var superTranslate = i18next_1.default.t.bind(i18next_1.default);
i18next_1.default.init({
    lng: 'en',
    initImmediate: true,
    resources: {
        en: { translation: getTranslationResource('en') },
        he: { translation: getTranslationResource('he') },
    },
});
// Gets a locale and returns the full resource object
function getTranslationResource(locale) {
    var paths = paths_1.Paths;
    if (process.env.TORTILLA_CWD) {
        paths = paths_1.Paths.resolveProject(process.env.TORTILLA_CWD);
    }
    paths = [
        // Static locales
        Path.resolve(paths.tortilla.translator.locales, locale + ".json"),
        // User defined locales
        Path.resolve(paths.locales, locale + ".json"),
    ];
    // Unite all resources and return a single one
    return paths.reduce(function (resource, path) {
        // Clear cache so files can be properly reloaded
        try {
            delete require.cache[require.resolve(path)];
            var extension = require(path);
            utils_1.Utils.merge(resource, extension);
        }
        catch (err) {
            // nothing to do here...
        }
        return resource;
    }, {});
}
// Returns i18n translation wrapped with some extra functionality
function translate() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var result = superTranslate.apply(void 0, args);
    return result && new translation_1.Translation(result);
}
// Any translation would be done using the provided locale
function scopeLanguage(language, fn) {
    if (!language) {
        return fn();
    }
    var oldLanguage = i18next_1.default.translator.language;
    try {
        i18next_1.default.translator.changeLanguage(language);
        fn();
    }
    finally {
        i18next_1.default.translator.changeLanguage(oldLanguage);
    }
}
// Shallow cloning i18n so it won't be changed
exports.Translator = utils_1.Utils.extend(i18next_1.default, {
    translate: translate,
    scopeLanguage: scopeLanguage,
});
//# sourceMappingURL=index.js.map